/** Bandpass Neuron
 *  
 *           \class  NeuronBP
 *
 *                   A bandpass neuron can emulate a filter
 *                   according to:\n
 *                   \f[
 *                   h(t)=\frac{1}{b} e^{at} \sin{(bt)}
 *                   \f]
 *                   with \f$a=\frac{\pi f}{q}\f$
 *                   and \f$b=\sqrt{(2 \pi f)^2-a^2}\f$ yields
 *                   into:
 *                   \f[
 *                   y[(n+1)T] =
 *                   -2 \cdot e^{-a T} cos{b T} y[nY] -
 *                   e^{-2 \cdot a T} y[(n-1)T ]+
 *                   \frac{1}{b}e^{aT}\sin{(bT)} x[nT]
 *                   \f]
 *                   The last factor \f$x[n T]\f$ is set to 1
 *                   (irrelevant because of the normalization) and T
 *                   is set to 1, too.\n
 *                   
 *                   \b f (default) = 0.1 \n
 *                   \b q (default) = 0.51 \n
 *                   \b normlize (default) = true \n
 *  
 *                   The higher the frequency the sharper the peak\n
 *                   The higher the quality the more is the filter
 *                   oscillating
 *  
 *            \date  01/03/2007 09:26:11 PM CET
 *
 *         \version  1.0
 *          \author  Christoph Kolodziejski (kolo), kolo@bccn-goettingen.de
 *                   BCCN Goettingen
 *  
 */

// =====================================================================================
// Includes
// =====================================================================================

#include "NeuronBP.h"

// =====================================================================================
// defines and consts
// =====================================================================================

#define max(a,b) (((a) > (b)) ? (a) : (b))

// =====================================================================================
// Constructor and Destructor
// =====================================================================================

/** Constructor that sets frequency and quality.
 *
 *              Constructor that sets frequency and quality.
 *
 *      @param  f double - The frequency.
 *      @param  q double - The quality.
 *
 */
NeuronBP::NeuronBP(double f, double q)
{
  reset();
  normalize_ = true;
  burst_ = false;
  setFQ(f,q);
}


// =====================================================================================
// =====================================================================================

/** Initialization
 *
 *              Initialization of the \b denomiator_ needed for the
 *              calculation of the filter.
 *
 *      @param  f double - The frequency
 *      @param  q double - The quality
 *     @return   -
 *
 *    @remarks  The quality should be bigger than 0.51
 */
void NeuronBP::setFQ(double f, double q)
{
  // If Q is ok
  if (q > 0)
  { 

    double fTimesPi = f * M_PI * 2.;
    double e = fTimesPi / (2. * q);

    // If root is ok
    if ((fTimesPi * fTimesPi - e * e) > 0)
    {

      double w = sqrt(fTimesPi * fTimesPi - e * e);
      denominator_[0] = -2. * exp(-e) * cos(w);
      denominator_[1] = exp(-2. * e);

      calcNorm();

    }
    // If root is bac
    else
    { 
      std::cerr << "ERROR - bandpass: The quality is too low: f-e =" <<(fTimesPi * fTimesPi - e * e)<< std::endl;
      exit(1);
    }

  }
  // If Q is bad
  else
  { 
    std::cerr << "bandpass: The quality is rubbish!" << std::endl;
    exit(1);
  }
}

/** Calculation of the normalizing factor.
 *
 *              Calculation of the normalizing factor.
 *
 *     @return   -
 *
 *    @remarks  The 'search' for the maximum is limited to 200. This
 *              can cause trouble if the frequency is to low.
 */
void NeuronBP::calcNorm()
{
  norm_ = 1.;
  // Reset of buffer
  for (int i = 0; i < 2; i++)
  {
    buffer_[i] = 0;
  }
  // Calculation of new norm
  double nnorm = 0;
  for (int i = 0; i < 1000; i++)
  {
    if(burst_)
      calculate(1);
    else
      calculate((i == 5) ? 1 : 0);

    nnorm = max(nnorm, nextoutput_);
  }
  nextoutput_ = 0;

  if (nnorm != 0)
    norm_ = nnorm;

  // Reset of buffer
  for (int i = 0; i < 2; i++)
  {
    buffer_[i] = 0;
  }
}

/** Calculation of the next value
 *
 *             Calculation of the next value according to the filter
 *             equations. 
 *
 *      @param  in double - The new value.
 *     @return   -
 *
 *    @remarks   -
 */
void NeuronBP::calculate(double in)
{

  nextoutput_ = in 
              - denominator_[0] * buffer_[0] 
              - denominator_[1] * buffer_[1];

  buffer_[1] = buffer_[0];
  buffer_[0] = nextoutput_;


  if (normalize_)
  {
    nextoutput_ /= norm_;
  }


}


/** Reseting of the neuron.
 *
 *             Reseting of the neuron. 
 *
 *     @return   -
 *
 *    @remarks   -
 */
void NeuronBP::reset()
{
  buffer_[0] = 0;
  buffer_[1] = 0;

  output_ = 0;
  nextoutput_ = 0;
}

