/** Normalized ICO Neuron.
 *  
 *           \class  NeuronNICO
 *
 *                   A normalized ICO Neuron that inherit from Neuron
 *                   or ICO Neuron, respectively.\n
 *                   Learning rule:
 *                   \f[
 *                   \Delta \rho_i(t) = \mu \cdot N(u_i(t)) \cdot u_0^{'}(t) 
 *                   \f]
 *                   where \f$\mu\f$ is the learning rate \n
 *                   and \f$\rho_i\f$ is the weight \n
 *                   and \f$u_i\f$ with \f$i=1..n\f$ are the
 *                   predictive inputs\n
 *                   and \f$N()\f$ is a normalizing function\n
 *                   and \f$u_0\f$ is the reflex input.
 *                   \n
 *                   \b learning rate (default) = 1e-4
 *  
 *            \date  01/03/2007 08:28:12 PM CET
 *
 *         \version  1.0
 *          \author  Christoph Kolodziejski (kolo), kolo@bccn-goettingen.de
 *                   BCCN Goettingen
 *  
 */

// =====================================================================================
// Includes
// =====================================================================================

#include "NeuronNICO.h"
#include "Synapse.h"

// =====================================================================================
// defines and consts
// =====================================================================================


// =====================================================================================
// =====================================================================================


/** Calculation of the next output.
 *
 *              Iterates all connected synapses and calculates the \b
 *              nextoutput_ according to the normalized ICO rule.
 *              
 *     @return   -
 *
 *    @remarks  The flags are: \n
 *              \b 0 <=> Reflex input (will not be learned) \n
 *              \b 1 <=> Predictive input (will be learned) \n
 *              \b 2 <=> Normalized predictive input (will not be learned)
 */
void NeuronNICO::calculate()
{

  // Iterate all synapses and sum them up into nextoutput_
  nextoutput_ = 0;
  double nextreflex = 0;
  double normalizedpreflex = 0;
  std::list<Synapse*>::iterator i, iend = synapsis_.end(); 
  for(i = synapsis_.begin(); i!=iend; ++i)
  {
    nextoutput_ += (*i)->getWeight() * (*i)->getOutput();
    if ((*i)->getFlags() == 0)
    {
      nextreflex += (*i)->getOutput();
    }


    if ((*i)->getFlags() == 2)
    {
      normalizedpreflex = (*i)->getOutput();
    }
  }

  if(noLearning_)
    return;

  // Learn
  double derivReflex = nextreflex - reflex_;

  for(i = synapsis_.begin(); i!=iend; ++i)
  {
    if ((*i)->getFlags() == 1)
    {
      (*i)->setWeight((*i)->getWeight() + learningRate_ * derivReflex
            * normalizedpreflex);

    }
  }
  reflex_ = nextreflex;
}
