/*
  Pololu3pi.h - Library for using the 3pi robot.
*/

/*
 * Written by Paul Grayson, 2008.
 * Copyright (c) 2008 Pololu Corporation. For more information, see
 *
 *   http://www.pololu.com
 *   http://forum.pololu.com
 *   http://www.pololu.com/docs/0J18/6
 *
 * You may freely modify and share this code, as long as you keep this
 * notice intact (including the two links above).  Licensed under the
 * Creative Commons BY-SA 3.0 license:
 *
 *   http://creativecommons.org/licenses/by-sa/3.0/
 *
 * Disclaimer: To the extent permitted by law, Pololu provides this work
 * without any warranty.  It might be defective, in which case you agree
 * to be responsible for all resulting costs and damages.
 */
 
#ifndef Neuron_h
#define Neuron_h

// =====================================================================================
// System Includes
// =====================================================================================

#include <list>
#include <iostream>

// =====================================================================================
// Project Includes
// =====================================================================================

//#include "Recordable.h"
//#include "ActivationFunction.h"

// =====================================================================================
// Forward class declarations
// =====================================================================================

class Synapse;


// =====================================================================================
// =====================================================================================
class Neuron : public Recordable
{

  public:

    // ====================  LIFECYCLE   ========================================= 

    /*! Constructor */
    Neuron ();
    Neuron (double bias);

    /*! Destructor */
    virtual ~Neuron ();

    // ====================  OPERATORS   ========================================= 

    // ====================  OPERATIONS  ========================================= 

    /** Add a synapse to this neuron.
     *   
     *              Add a synapse to this neuron
     *
     *      @param  synapse Synapse* - A pointer to a synapse that will be connected
     *
     */
    void addSynapse(Synapse* synapse);

    /** Reset the neuron to zero.
     *   
     * Variable \b output_ and \b nextoutput_ are set 0
     *
     */
    virtual void reset();

    /** Calculation of the next output
     *
     *              Iterating the connected synapses and summing up
     *              their output weighted with their weight
     *              The output is stored into the variable \b nextoutput_
     *
     *    @remarks  Use always the variable \b nextoutput_ 
     */
    virtual void calculate();

    /** Updating after calculation
     *
     *               After all neurons within the Neuronal Network are
     *               calculated they become updated
     *               \b nextoutput_ -> \b output_
     *
     */
    virtual void update();

    /** Destruction of the neuron
     *
     *              Destruction of all connected synapses and the
     *              neuron itself
     *
     *    @remarks  The complete neural network destruct itself
     *              recursive thus you don't have to do this by hand 
     *
     */
    void clear();

    // ====================  ACCESS      ========================================= 

    void setActivationFunction(ActivationFunction* activation) {delete activation_; 
                                                                activation_ = activation;};
    ActivationFunction* getActivationFunction() {return activation_;};

    void setBias(double bias) {bias_ = bias;};
    double getBias() {return bias_;};

    void setRandomBias(bool randomBias) {randomBias_ = randomBias;};

    void setOutput(double output) {output_ = output;};
    double getOutput() {return output_;};

    double record(){return output_;}

    void setIsNonNegative(bool isNonNegative) {isNonNegative_ = isNonNegative;}; 

    // ====================  INQUIRY     ========================================= 

  protected:
    /*! A list of synapses */
    std::list<Synapse*> synapsis_;

    /*! The output of the neuron */
    double output_;

    /*! 
     *  The next output of the neuron that is determined during the
     *  calculating step and set to output during the update step
     */
    double nextoutput_;  

    /*! The Bias of the neuron */
    double bias_;

    /*! Flag if the bias is uniform random between 0 and bias*/
    double randomBias_;

    /*! The activation function of the neuron */
    bool isNonNegative_;

    /*! The activation function of the neuron */
    ActivationFunction* activation_;

  private:

};

#endif

#endif

// Local Variables: **
// mode: C++ **
// c-basic-offset: 4 **
// tab-width: 4 **
// indent-tabs-mode: t **
// end: **
