/** Bandpass Neuron
 *  
 *           \class  NeuronBP
 *
 *                   A bandpass neuron can emulate a filter
 *                   according to:\n
 *                   \f[
 *                   h(t)=\frac{1}{b} e^{at} \sin{(bt)}
 *                   \f]
 *                   with \f$a=\frac{\pi f}{q}\f$
 *                   and \f$b=\sqrt{(2 \pi f)^2-a^2}\f$ yields
 *                   into:
 *                   \f[
 *                   y[(n+1)T] =
 *                   -2 \cdot e^{-a T} cos{b T} y[nY] -
 *                   e^{-2 \cdot a T} y[(n-1)T ]+
 *                   \frac{1}{b}e^{aT}\sin{(bT)} x[nT]
 *                   \f]
 *                   The last factor \f$x[n T]\f$ is set to 1
 *                   (irrelevant because of the normalization) and T
 *                   is set to 1, too.\n
 *                   
 *                   \b f (default) = 0.1 \n
 *                   \b q (default) = 0.51 \n
 *                   \b normlize (default) = true \n
 *  
 *                   The higher the frequency the sharper the peak\n
 *                   The higher the quality the more is the filter
 *                   oscillating
 *  
 *            \date  01/03/2007 09:26:11 PM CET
 *
 *         \version  1.0
 *          \author  Christoph Kolodziejski (kolo), kolo@bccn-goettingen.de
 *                   BCCN Goettingen
 *  
 */

#ifndef NeuronBP_h_
#define NeuronBP_h_

// =====================================================================================
// System Includes
// =====================================================================================


#include <math.h>


#ifndef OrangutanLCD_h
#include <pololu/OrangutanLCD.h>
#endif




// =====================================================================================
// Forward class declarations
// =====================================================================================


// =====================================================================================
// =====================================================================================
class NeuronBP
{

  public:

    // ====================  LIFECYCLE   ========================================= 

    /*! Constructor */
    NeuronBP(double f=DEF_F, double q=DEF_Q);

    // ====================  OPERATORS   ========================================= 

    // ====================  OPERATIONS  ========================================= 


    /** Calculation of the next value
     *
     *             Calculation of the next value according to the filter
     *             equations. 
     *
     *      @param  in double - The new value.
     *
     */
    void calculate(double in);

    /** Updating of the neuron.
     *
     *              Updating of the neuron either without recruitment
     *              (nothing changed compared to Neuron) or with
     *              recruitment: The calculated value is put as the
     *              normalized one into the learning pathway (\b
     *              normalized \b output) and the output weighted with the
     *              fibres is set as 'real' output. \n
     *              Because of the feedback the number of used fibres may
     *              have changed.
     *
     */
    void update();

    /** Reseting of the neuron.
     *
     *             Reseting of the neuron. 
     *
     */
    void reset();

    /** Initialization
     *
     *              Initialization of the \b denomiator_ needed for the
     *              calculation of the filter.
     *
     *      @param  f double - The frequency
     *      @param  q double - The quality
     *
     *    @remarks  The quality should be bigger than 0.51
     */
    void setFQ(double f, double q);

    /** Calculation of the normalizing factor.
     *
     *              Calculation of the normalizing factor.
     *
     *    @remarks  The 'search' for the maximum is limited to 200. This
     *              can cause trouble if the frequency is to low.
     */
    void calcNorm();
    
    // ====================  ACCESS      ========================================= 


    void setBurst(bool burst) {burst_ = burst; calcNorm();};

    void setNormalize(bool fnormalize) {normalize_ = fnormalize;};
    bool getNormalize() {return normalize_;};

    // ====================  INQUIRY     ========================================= 

  private:
    /*! The pre-factor */
    double  denominator_[2];

    /*! The output with history */
    double  buffer_[2];

    double nextoutput_;
    double output_;

    /*! The normalizing factor */
    double  norm_;

    /*! True if normalizing bursts */
    bool    burst_;

    /*! A switch for normalizing */
    bool    normalize_;

    static const double  DEF_F = 0.1;
    static const double  DEF_Q = 0.51;

};

#endif
